--[[ CombatTime ]]

if not FixR then return end

local DefaultO = {
  ["framePoint"] = "CENTER";
  ["frameRelativeTo"] = "UIParent";
  ["frameRelativePoint"] = "CENTER";
  ["frameOffsetX"] = 0;
  ["frameOffsetY"] = 0;
}
local misc = {
  ["desc"] = "Display combat length";
  ["alpha"] = true;
  ["GUI"] = {
    [1] = {
      ["type"] = "Button";
      ["opt"] = "toggle moving";
      ["func"] = function(self) self:SetOption("move"); end;
    };
    [2] = {
      ["type"] = "Button";
      ["opt"] = "reset position";
      ["func"] = function(self) self:SetOption("reset"); end;
    };
  };
}

local M = FixR:NewModule("CombatTime", DefaultO, misc)

local frame = nil
local frameEvents = {};
local moving = false
local secondsInCombat = 0

local function initFrame()
  frame = CreateFrame("Frame", "FixR_CombatTimeFrame", UIParent)
  frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  frame:SetFrameStrata("HIGH")
  frame:SetSize(220, 40)
  frame:SetScript("OnEvent", function(self, event, ...)
    frameEvents[event](self, ...)
  end);
  frame:SetScript("OnDragStart", function(self) self:StartMoving(); end);
  frame:SetScript("OnDragStop", function(self)
    self:StopMovingOrSizing()
    point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()
    M.O["framePoint"] = point or "CENTER"
    M.O["frameRelativeTo"] = relativeTo or "UIParent"
    M.O["frameRelativePoint"] = relativePoint or "CENTER"
    M.O["frameOffsetX"] = FixR:Round(xOfs, 4)
    M.O["frameOffsetY"] = FixR:Round(yOfs, 4)
  end);
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  frame.timeSinceLastUpdate = 0
  frame.inCombat = false
  
  frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
  frame.text:SetPoint("RIGHT")
  frame.text:SetText("CombatTime")
  frame.text:SetTextColor(1, 1, 1, 1)
  
  frame.OnUpdate = function(self, elapsed)
    self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed
    while (self.inCombat and self.timeSinceLastUpdate > 1) do --in seconds
      secondsInCombat = secondsInCombat + 1
      self.updateDisplay()
      self.timeSinceLastUpdate = self.timeSinceLastUpdate - 1
    end
  end
  frame.updateDisplay = function()
    local l = secondsInCombat
    local d = math.floor(l / 86400)
    l = l - 86400 * d
    local h = math.floor(l / 3600)
    l = l - 3600 * h
    local m = math.floor(l / 60)
    l = l - 60 * m
    local s = l
    if d > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      frame.text:SetText(string.format("%dd %d:%s:%s", d, h, m, s))
    elseif h > 0 then
      if m < 10 then
        m = "0"..m
      end
      if s < 10 then
        s = "0"..s
      end
      frame.text:SetText(string.format("%d:%s:%s", h, m, s))
    elseif m > 0 then
      if s < 10 then
        s = "0"..s
      end
      frame.text:SetText(string.format("%d:%s", m, s))
    else
      frame.text:SetText(s)
    end
  end
end

function frameEvents:PLAYER_REGEN_DISABLED(...)
  self.inCombat = true
  secondsInCombat = 0
  self.timeSinceLastUpdate = 0
  self.updateDisplay()
end
function frameEvents:PLAYER_REGEN_ENABLED(...)
  self.inCombat = false
  self.updateDisplay()
end



function M:Load()
  if not frame then
    initFrame()
  end
  for k, v in pairs(frameEvents) do
    frame:RegisterEvent(k);
  end
  frame:Show()
end

function M:Unload()
  if frame then
    for k, v in pairs(frameEvents) do
      frame:UnregisterEvent(k);
    end
    frame:Hide()
  end
end

function M:SetOption(arg1, arg2)
  if arg1 == "move" then
    if moving then
      moving = false
      --frame.texture:SetTexture(0.67, 0.67, 1, 0.01) -- cannot :Hide() in combat, so make it nearly transparent
      frame:SetMovable(false) --click-through
      frame:EnableMouse(false)
      frame:RegisterForDrag("")
    else
      moving = true
      --frame.texture:SetTexture(0.67, 0.67, 1, 1) --opaque
      frame:SetMovable(true);
      frame:EnableMouse(true);
      frame:RegisterForDrag("LeftButton");
    end
    FixR:Echo("CombatTime: now "..(moving and "moving" or "fixed"))
  elseif arg1 == "reset" then
    M.O["framePoint"] = DefaultO["framePoint"]
    M.O["frameRelativeTo"] = DefaultO["frameRelativeTo"]
    M.O["frameRelativePoint"] = DefaultO["frameRelativePoint"]
    M.O["frameOffsetX"] = DefaultO["frameOffsetX"]
    M.O["frameOffsetY"] = DefaultO["frameOffsetY"]
    
    frame:ClearAllPoints()
    frame:SetPoint(M.O["framePoint"], M.O["frameRelativeTo"], M.O["frameRelativePoint"], M.O["frameOffsetX"], M.O["frameOffsetY"])
  end
end

function M:ShowOptions()
end
